# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import lilypondnametools
from abjad.tools import markuptools
from abjad.tools.spannertools.Spanner import Spanner
from abjad.tools.topleveltools import inspect_
from abjad.tools.topleveltools import new


class TempoSpanner(Spanner):
    r'''Tempo spanner.

    ..  container:: example

        With tempo indicators only:

        ::

            >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[0], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 72)
            >>> attach(tempo, staff[3], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[5], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
            } <<
                \new Staff {
                    \time 3/8
                    c'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                    d'8.
                    e'4. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    g'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                72
                            }
                        }
                    f'8.
                    ef'4. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                }
            >>

    ..  container:: example

        With an accelerando:

        ::

            >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[0], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 72)
            >>> attach(tempo, staff[3], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[5], is_annotation=True)

        ::

            >>> accelerando = indicatortools.Accelerando()
            >>> attach(accelerando, staff[0], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        accel.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        \hspace
                            #1.25
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    c'8. \startTextSpan
                    d'8.
                    e'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    g'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                72
                            }
                        }
                    f'8.
                    ef'4. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                }
            >>

    ..  container:: example

        With a ritardando:

        ::

            >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[0], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 72)
            >>> attach(tempo, staff[3], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[5], is_annotation=True)

        ::

            >>> ritardando = indicatortools.Ritardando()
            >>> attach(ritardando, staff[3], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    c'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                    d'8.
                    e'4. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        rit.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                72
                            }
                        \hspace
                            #1.25
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    g'8. \startTextSpan
                    f'8.
                    ef'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                }
            >>

    ..  container:: example

        With both an accelerando and a ritardando:

        ::

            >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[0], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 72)
            >>> attach(tempo, staff[3], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[5], is_annotation=True)

        ::

            >>> accelerando = indicatortools.Accelerando()
            >>> attach(accelerando, staff[0], is_annotation=True)
            >>> ritardando = indicatortools.Ritardando()
            >>> attach(ritardando, staff[3], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        accel.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        \hspace
                            #1.25
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    c'8. \startTextSpan
                    d'8.
                    e'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        rit.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                72
                            }
                        \hspace
                            #1.25
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    g'8. \startTextSpan
                    f'8.
                    ef'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                }
            >>

    ..  container:: example

        Implicit start to (music-initial) accelerando:

        ::

            >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 72)
            >>> attach(tempo, staff[3], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[5], is_annotation=True)

        ::

            >>> accelerando = indicatortools.Accelerando()
            >>> attach(accelerando, staff[0], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        accel.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \large
                            \upright
                                accel.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    c'8. \startTextSpan
                    d'8.
                    e'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    g'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                72
                            }
                        }
                    f'8.
                    ef'4. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                }
            >>

    ..  container:: example

        Implicit start to (music-initial) ritardando:

        ::

            >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 72)
            >>> attach(tempo, staff[3], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[5], is_annotation=True)

        ::

            >>> ritardando = indicatortools.Ritardando()
            >>> attach(ritardando, staff[0], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        rit.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \large
                            \upright
                                rit.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    c'8. \startTextSpan
                    d'8.
                    e'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    g'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                72
                            }
                        }
                    f'8.
                    ef'4. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                }
            >>

    ..  container:: example

        Implicit start to (midmusic) accelerando:

        ::

            >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[0], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 120)
            >>> attach(tempo, staff[5], is_annotation=True)

        ::

            >>> accelerando = indicatortools.Accelerando()
            >>> attach(accelerando, staff[3], is_annotation=True)

        ::

            >>> tempo_spanner = spannertools.TempoSpanner(
            ...     start_with_parenthesized_tempo=True,
            ...     )
            >>> attach(tempo_spanner, staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    c'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                    d'8.
                    e'4. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        accel.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \line
                                    {
                                        \fontsize
                                            #-6
                                            \general-align
                                                #Y
                                                #DOWN
                                                \note-by-number
                                                    #2
                                                    #0
                                                    #1
                                        \upright
                                            {
                                                =
                                                90
                                            }
                                    }
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    g'8. \startTextSpan
                    f'8.
                    ef'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                120
                            }
                        }
                }
            >>

    ..  container:: example

        Implicit start to (midmusic) ritardando:

        ::

            >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[0], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[5], is_annotation=True)

        ::

            >>> ritardando = indicatortools.Ritardando()
            >>> attach(ritardando, staff[3], is_annotation=True)

        ::

            >>> tempo_spanner = spannertools.TempoSpanner(
            ...     start_with_parenthesized_tempo=True,
            ...     )
            >>> attach(tempo_spanner, staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    c'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                    d'8.
                    e'4. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        rit.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \line
                                    {
                                        \fontsize
                                            #-6
                                            \general-align
                                                #Y
                                                #DOWN
                                                \note-by-number
                                                    #2
                                                    #0
                                                    #1
                                        \upright
                                            {
                                                =
                                                90
                                            }
                                    }
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    g'8. \startTextSpan
                    f'8.
                    ef'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                }
            >>

    ..  container:: example

        With an accelerando over a line break:

        ::

            >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])
            >>> command = indicatortools.LilyPondCommand('break', 'after')
            >>> attach(command, staff[3])

        ::

            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[6], is_annotation=True)

        ::

            >>> accelerando = indicatortools.Accelerando()
            >>> attach(accelerando, staff[2], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    c'4.
                    d'4.
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        accel.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        \hspace
                            #1.25
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    e'4. \startTextSpan
                    f'4.
                    \break
                    g'4.
                    a'4.
                    b'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    c''4.
                }
            >>

    ..  container:: example

        With a ritardando over a line break:

        ::

            >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])
            >>> command = indicatortools.LilyPondCommand('break', 'after')
            >>> attach(command, staff[3])

        ::

            >>> tempo = Tempo(Duration(1, 4), 90)
            >>> attach(tempo, staff[2], is_annotation=True)
            >>> tempo = Tempo(Duration(1, 4), 60)
            >>> attach(tempo, staff[6], is_annotation=True)

        ::

            >>> ritardando = indicatortools.Ritardando()
            >>> attach(ritardando, staff[2], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    c'4.
                    d'4.
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        rit.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        \hspace
                            #1.25
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    e'4. \startTextSpan
                    f'4.
                    \break
                    g'4.
                    a'4.
                    b'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #0
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        }
                    c''4.
                }
            >>

    ..  container:: example

        With a metric modulation:

        ::

            >>> staff = Staff("c'8. d'8. e'4. g'8. f'8. ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(3, 4), 90)
            >>> attach(tempo, staff[0], is_annotation=True)
            >>> tempo = Tempo(Duration(3, 4), 60)
            >>> attach(tempo, staff[3], is_annotation=True)
            >>> metric_modulation = indicatortools.MetricModulation(
            ...     left_rhythm=Note('c4.'),
            ...     right_rhythm=Note('c4'),
            ...     )
            >>> attach(metric_modulation, staff[3], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    c'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #1
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    d'8.
                    e'4.
                    g'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #1
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        \hspace
                            #0.5
                        \general-align
                            #Y
                            #DOWN
                            \override
                                #'(padding . 0.5)
                                \parenthesize
                                    \line
                                        {
                                            \scale
                                                #'(0.5 . 0.5)
                                                \score
                                                    {
                                                        \new Score \with {
                                                            \override SpacingSpanner.spacing-increment = #0.5
                                                            proportionalNotationDuration = ##f
                                                        } <<
                                                            \new RhythmicStaff \with {
                                                                \remove Time_signature_engraver
                                                                \remove Staff_symbol_engraver
                                                                \override Stem.direction = #up
                                                                \override Stem.length = #5
                                                                \override TupletBracket.bracket-visibility = ##t
                                                                \override TupletBracket.direction = #up
                                                                \override TupletBracket.padding = #1.25
                                                                \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                                                \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                                                tupletFullLength = ##t
                                                            } {
                                                                c4.
                                                            }
                                                        >>
                                                        \layout {
                                                            indent = #0
                                                            ragged-right = ##t
                                                        }
                                                    }
                                            =
                                            \hspace
                                                #-0.5
                                            \scale
                                                #'(0.5 . 0.5)
                                                \score
                                                    {
                                                        \new Score \with {
                                                            \override SpacingSpanner.spacing-increment = #0.5
                                                            proportionalNotationDuration = ##f
                                                        } <<
                                                            \new RhythmicStaff \with {
                                                                \remove Time_signature_engraver
                                                                \remove Staff_symbol_engraver
                                                                \override Stem.direction = #up
                                                                \override Stem.length = #5
                                                                \override TupletBracket.bracket-visibility = ##t
                                                                \override TupletBracket.direction = #up
                                                                \override TupletBracket.padding = #1.25
                                                                \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                                                \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                                                tupletFullLength = ##t
                                                            } {
                                                                c4
                                                            }
                                                        >>
                                                        \layout {
                                                            indent = #0
                                                            ragged-right = ##t
                                                        }
                                                    }
                                        }
                        }
                    f'8.
                    ef'4.
                }
            >>

    ..  container:: example

        With a metric modulation and an accelerando:

        ::

            >>> staff = Staff("c'8. d'8. e'4. g'8. f'8. ef'4.")
            >>> attach(TimeSignature((3, 8)), staff)
            >>> score = Score([staff])

        ::

            >>> tempo = Tempo(Duration(3, 4), 90)
            >>> attach(tempo, staff[0], is_annotation=True)
            >>> tempo = Tempo(Duration(3, 4), 60)
            >>> attach(tempo, staff[3], is_annotation=True)
            >>> metric_modulation = indicatortools.MetricModulation(
            ...     left_rhythm=Note('c4.'),
            ...     right_rhythm=Note('c4'),
            ...     )
            >>> attach(metric_modulation, staff[3], is_annotation=True)
            >>> attach(Accelerando(), staff[3], is_annotation=True)
            >>> tempo = Tempo(Duration(3, 4), 90)
            >>> attach(tempo, staff[-1], is_annotation=True)

        ::

            >>> attach(spannertools.TempoSpanner(), staff[:])

        ::

            >>> override(score).text_script.staff_padding = 2.25
            >>> override(score).text_spanner.staff_padding = 3

        ::

            >>> show(score) # doctest: +SKIP

        ..  doctest::

            >>> f(score)
            \new Score \with {
                \override TextScript.staff-padding = #2.25
                \override TextSpanner.staff-padding = #3
            } <<
                \new Staff {
                    \time 3/8
                    c'8. ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #1
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                    d'8.
                    e'4.
                    \once \override TextSpanner.arrow-width = 0.25
                    \once \override TextSpanner.bound-details.left-broken.padding = -2
                    \once \override TextSpanner.bound-details.left-broken.text = \markup {
                        \override
                            #'(padding . 0.45)
                            \parenthesize
                                \large
                                    \upright
                                        accel.
                        \hspace
                            #0.75
                        }
                    \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                    \once \override TextSpanner.bound-details.left.text = \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #1
                                    #1
                        \upright
                            {
                                =
                                60
                            }
                        \hspace
                            #0.5
                        \general-align
                            #Y
                            #DOWN
                            \override
                                #'(padding . 0.5)
                                \parenthesize
                                    \line
                                        {
                                            \scale
                                                #'(0.5 . 0.5)
                                                \score
                                                    {
                                                        \new Score \with {
                                                            \override SpacingSpanner.spacing-increment = #0.5
                                                            proportionalNotationDuration = ##f
                                                        } <<
                                                            \new RhythmicStaff \with {
                                                                \remove Time_signature_engraver
                                                                \remove Staff_symbol_engraver
                                                                \override Stem.direction = #up
                                                                \override Stem.length = #5
                                                                \override TupletBracket.bracket-visibility = ##t
                                                                \override TupletBracket.direction = #up
                                                                \override TupletBracket.padding = #1.25
                                                                \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                                                \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                                                tupletFullLength = ##t
                                                            } {
                                                                c4.
                                                            }
                                                        >>
                                                        \layout {
                                                            indent = #0
                                                            ragged-right = ##t
                                                        }
                                                    }
                                            =
                                            \hspace
                                                #-0.5
                                            \scale
                                                #'(0.5 . 0.5)
                                                \score
                                                    {
                                                        \new Score \with {
                                                            \override SpacingSpanner.spacing-increment = #0.5
                                                            proportionalNotationDuration = ##f
                                                        } <<
                                                            \new RhythmicStaff \with {
                                                                \remove Time_signature_engraver
                                                                \remove Staff_symbol_engraver
                                                                \override Stem.direction = #up
                                                                \override Stem.length = #5
                                                                \override TupletBracket.bracket-visibility = ##t
                                                                \override TupletBracket.direction = #up
                                                                \override TupletBracket.padding = #1.25
                                                                \override TupletBracket.shorten-pair = #'(-1 . -1.5)
                                                                \override TupletNumber.text = #tuplet-number::calc-fraction-text
                                                                tupletFullLength = ##t
                                                            } {
                                                                c4
                                                            }
                                                        >>
                                                        \layout {
                                                            indent = #0
                                                            ragged-right = ##t
                                                        }
                                                    }
                                        }
                        \hspace
                            #1.25
                        }
                    \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                    \once \override TextSpanner.bound-details.right-broken.padding = 0
                    \once \override TextSpanner.bound-details.right-broken.text = ##f
                    \once \override TextSpanner.bound-details.right.arrow = ##t
                    \once \override TextSpanner.bound-details.right.padding = 2
                    \once \override TextSpanner.bound-details.right.text = ##f
                    \once \override TextSpanner.dash-fraction = 0.25
                    \once \override TextSpanner.dash-period = 1.5
                    g'8. \startTextSpan
                    f'8.
                    ef'4. \stopTextSpan ^ \markup {
                        \fontsize
                            #-6
                            \general-align
                                #Y
                                #DOWN
                                \note-by-number
                                    #2
                                    #1
                                    #1
                        \upright
                            {
                                =
                                90
                            }
                        }
                }
            >>

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_left_broken_padding',
        '_left_broken_text',
        '_start_with_parenthesized_tempo',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        left_broken_padding=None,
        left_broken_text=None,
        start_with_parenthesized_tempo=False,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            overrides=overrides,
            )
        assert isinstance(left_broken_padding, (int, float, type(None)))
        self._left_broken_padding = left_broken_padding
        prototype = (markuptools.Markup, type(None))
        assert isinstance(left_broken_text, prototype)
        self._left_broken_text = left_broken_text
        assert isinstance(start_with_parenthesized_tempo, (bool, type(None)))
        self._start_with_parenthesized_tempo = start_with_parenthesized_tempo

    ### PRIVATE METHODS ###

    def _combine_tempo_and_metric_modulation(
        self,
        tempo,
        metric_modulation,
        ):
        assert tempo is not None or metric_modulation is not None
        if tempo is None:
            return metric_modulation._get_markup(
                music_scale_pair=(0.5, 0.5),
                )
        if metric_modulation is None:
            return tempo._to_markup()
        tempo_markup = tempo._to_markup()
        tempo_markup = tempo_markup + tempo_markup.hspace(0.5)
        modulation_markup = metric_modulation._get_markup(
            music_scale_pair=(0.5, 0.5),
            )
        modulation_markup = modulation_markup.line([modulation_markup])
        modulation_markup = modulation_markup.parenthesize()
        modulation_markup = modulation_markup.override(('padding', 0.5))
        modulation_markup = modulation_markup.general_align('Y', Down)
        markup = tempo_markup + modulation_markup
        return markup

    def _get_annotations(self, leaf):
        inspector = inspect_(leaf)
        tempo = None
        prototype = indicatortools.Tempo,
        if inspector.has_indicator(prototype):
            tempo = inspector.get_indicator(prototype)
        tempo_trend = None
        prototype = (
            indicatortools.Accelerando,
            indicatortools.Ritardando,
            )
        if inspector.has_indicator(prototype):
            tempo_trend = inspector.get_indicator(prototype)
        metric_modulation = None
        prototype = indicatortools.MetricModulation
        if inspector.has_indicator(prototype):
            metric_modulation = inspector.get_indicator(prototype)
        return (
            tempo,
            tempo_trend,
            metric_modulation,
            )

    def _get_lilypond_format_bundle(self, leaf):
        lilypond_format_bundle = self._get_basic_lilypond_format_bundle(leaf)
        current_annotations = self._get_annotations(leaf)
        current_tempo = current_annotations[0]
        current_tempo_trend = current_annotations[1]
        current_metric_modulation = current_annotations[2]
        if current_tempo is None and current_tempo_trend is None:
            return lilypond_format_bundle
        previous_annotations = self._get_previous_annotations(leaf)
        previous_tempo = previous_annotations[0]
        previous_tempo_trend = previous_annotations[1]
        previous_metric_modulation = previous_annotations[2]
        # stop any previous tempo trend
        if previous_tempo_trend:
            spanner_stop = r'\stopTextSpan'
            lilypond_format_bundle.right.spanner_stops.append(spanner_stop)
        # use markup if no tempo trend starts now
        if current_tempo_trend is None:
            markup = self._combine_tempo_and_metric_modulation(
                current_tempo,
                current_metric_modulation,
                )
            markup = new(markup, direction=Up)
            string = format(markup, 'lilypond')
            lilypond_format_bundle.right.markup.append(string)
            return lilypond_format_bundle
        # use spanner if tempo trend starts now
        spanner_start = r'\startTextSpan'
        lilypond_format_bundle.right.spanner_starts.append(spanner_start)
        if current_tempo or current_metric_modulation:
            self._start_tempo_trend_spanner_with_explicit_start(
                leaf,
                lilypond_format_bundle,
                current_tempo,
                current_metric_modulation,
                )
        else:
            self._start_tempo_trend_spanner_with_implicit_start(
                leaf,
                lilypond_format_bundle,
                current_tempo_trend,
                previous_tempo,
                )
        #
        if self.left_broken_text is not None:
            markup = self.left_broken_text
        else:
            markup = current_tempo_trend._to_markup()
            markup = markup.parenthesize()
            markup = markup.override(('padding', 0.45))
            markup = markup + markup.hspace(0.75)
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'left-broken',
                'text',
                ),
            value=markup,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        self._make_other_text_spanner_overrides(lilypond_format_bundle)
        return lilypond_format_bundle

    def _get_previous_annotations(self, leaf):
        index = self._index(leaf)
        for index in reversed(range(index)):
            previous_leaf = self[index]
            annotations = self._get_annotations(previous_leaf)
            if any(_ is not None for _ in annotations):
                return annotations
        return None, None, None

    def _make_other_text_spanner_overrides(self, lilypond_format_bundle):
        r'''Alphabetically by property.
        '''
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'arrow-width',
                ),
            value=0.25,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'dash-fraction',
                ),
            value=0.25,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'dash-period',
                ),
            value=1.5,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'left',
                'stencil-align-dir-y',
                ),
            value=-0.5,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        if self.left_broken_padding is not None:
            padding = self.left_broken_padding
        else:
            padding = -2
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'left-broken',
                'padding',
                ),
            value=padding,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'right',
                'arrow',
                ),
            value=True,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'right',
                'padding',
                ),
            value=2,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'right',
                'text',
                ),
            value=False,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'right-broken',
                'arrow',
                ),
            value=False,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'right-broken',
                'padding',
                ),
            value=0,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)
        #
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'right-broken',
                'text',
                ),
            value=False,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)

    def _start_tempo_trend_spanner_with_explicit_start(
        self,
        leaf,
        lilypond_format_bundle,
        current_tempo,
        current_metric_modulation,
        ):
        #
        #markup = current_tempo._to_markup()
        markup = self._combine_tempo_and_metric_modulation(
            current_tempo,
            current_metric_modulation,
            )
        markup = markup + markup.hspace(1.25)
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'left',
                'text',
                ),
            value=markup,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)

    def _start_tempo_trend_spanner_with_implicit_start(
        self,
        leaf,
        lilypond_format_bundle,
        current_tempo_trend,
        previous_tempo,
        ):
        if self.start_with_parenthesized_tempo and previous_tempo:
            markup = previous_tempo._to_markup()
            markup = markup.line([markup])
            markup = markup.parenthesize()
            markup = markup.override(('padding', 0.45))
            markup = markup + markup.hspace(0.75)
        else:
            markup = current_tempo_trend._to_markup()
            markup = markup + markup.hspace(0.75)
        override_ = lilypondnametools.LilyPondGrobOverride(
            grob_name='TextSpanner',
            is_once=True,
            property_path=(
                'bound-details',
                'left',
                'text',
                ),
            value=markup,
            )
        override_string = override_.override_string
        lilypond_format_bundle.grob_overrides.append(override_string)

    ### PUBLIC PROPERTIES ###

    @property
    def left_broken_padding(self):
        r'''Gets left broken padding of tempo spanner.

        ..  container:: example

            With left broken padding set to none:

            ::

                >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
                >>> attach(TimeSignature((3, 8)), staff)
                >>> score = Score([staff])
                >>> command = indicatortools.LilyPondCommand('break', 'after')
                >>> attach(command, staff[3])

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> attach(tempo, staff[2], is_annotation=True)
                >>> tempo = Tempo(Duration(1, 4), 60)
                >>> attach(tempo, staff[6], is_annotation=True)

            ::

                >>> ritardando = indicatortools.Ritardando()
                >>> attach(ritardando, staff[2], is_annotation=True)

            ::

                >>> attach(spannertools.TempoSpanner(), staff[:])

            ::

                >>> override(score).text_script.staff_padding = 2.25
                >>> override(score).text_spanner.staff_padding = 3

            ::

                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score \with {
                    \override TextScript.staff-padding = #2.25
                    \override TextSpanner.staff-padding = #3
                } <<
                    \new Staff {
                        \time 3/8
                        c'4.
                        d'4.
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.padding = -2
                        \once \override TextSpanner.bound-details.left-broken.text = \markup {
                            \override
                                #'(padding . 0.45)
                                \parenthesize
                                    \large
                                        \upright
                                            rit.
                            \hspace
                                #0.75
                            }
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    90
                                }
                            \hspace
                                #1.25
                            }
                        \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right-broken.text = ##f
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 2
                        \once \override TextSpanner.bound-details.right.text = ##f
                        \once \override TextSpanner.dash-fraction = 0.25
                        \once \override TextSpanner.dash-period = 1.5
                        e'4. \startTextSpan
                        f'4.
                        \break
                        g'4.
                        a'4.
                        b'4. \stopTextSpan ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    60
                                }
                            }
                        c''4.
                    }
                >>

            Results in padding of ``-2``. (This is default behavior.)

        ..  container:: example

            With left broken padding set explicitly:

            ::

                >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
                >>> attach(TimeSignature((3, 8)), staff)
                >>> score = Score([staff])
                >>> command = indicatortools.LilyPondCommand('break', 'after')
                >>> attach(command, staff[3])

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> attach(tempo, staff[2], is_annotation=True)
                >>> tempo = Tempo(Duration(1, 4), 60)
                >>> attach(tempo, staff[6], is_annotation=True)

            ::

                >>> ritardando = indicatortools.Ritardando()
                >>> attach(ritardando, staff[2], is_annotation=True)

            ::

                >>> tempo_spanner = spannertools.TempoSpanner(
                ...     left_broken_padding=4,
                ...     )
                >>> attach(tempo_spanner, staff[:])

            ::

                >>> override(score).text_script.staff_padding = 2.25
                >>> override(score).text_spanner.staff_padding = 3

            ::

                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score \with {
                    \override TextScript.staff-padding = #2.25
                    \override TextSpanner.staff-padding = #3
                } <<
                    \new Staff {
                        \time 3/8
                        c'4.
                        d'4.
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.padding = 4
                        \once \override TextSpanner.bound-details.left-broken.text = \markup {
                            \override
                                #'(padding . 0.45)
                                \parenthesize
                                    \large
                                        \upright
                                            rit.
                            \hspace
                                #0.75
                            }
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    90
                                }
                            \hspace
                                #1.25
                            }
                        \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right-broken.text = ##f
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 2
                        \once \override TextSpanner.bound-details.right.text = ##f
                        \once \override TextSpanner.dash-fraction = 0.25
                        \once \override TextSpanner.dash-period = 1.5
                        e'4. \startTextSpan
                        f'4.
                        \break
                        g'4.
                        a'4.
                        b'4. \stopTextSpan ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    60
                                }
                            }
                        c''4.
                    }
                >>

        Returns number or none.
        '''
        return self._left_broken_padding

    @property
    def left_broken_text(self):
        r'''Gets left broken text of tempo spanner.

        ..  container:: example

            With left broken text set to none:

            ::

                >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
                >>> attach(TimeSignature((3, 8)), staff)
                >>> score = Score([staff])
                >>> command = indicatortools.LilyPondCommand('break', 'after')
                >>> attach(command, staff[3])

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> attach(tempo, staff[2], is_annotation=True)
                >>> tempo = Tempo(Duration(1, 4), 60)
                >>> attach(tempo, staff[6], is_annotation=True)

            ::

                >>> ritardando = indicatortools.Ritardando()
                >>> attach(ritardando, staff[2], is_annotation=True)

            ::

                >>> attach(spannertools.TempoSpanner(), staff[:])

            ::

                >>> override(score).text_script.staff_padding = 2.25
                >>> override(score).text_spanner.staff_padding = 3

            ::

                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score \with {
                    \override TextScript.staff-padding = #2.25
                    \override TextSpanner.staff-padding = #3
                } <<
                    \new Staff {
                        \time 3/8
                        c'4.
                        d'4.
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.padding = -2
                        \once \override TextSpanner.bound-details.left-broken.text = \markup {
                            \override
                                #'(padding . 0.45)
                                \parenthesize
                                    \large
                                        \upright
                                            rit.
                            \hspace
                                #0.75
                            }
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    90
                                }
                            \hspace
                                #1.25
                            }
                        \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right-broken.text = ##f
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 2
                        \once \override TextSpanner.bound-details.right.text = ##f
                        \once \override TextSpanner.dash-fraction = 0.25
                        \once \override TextSpanner.dash-period = 1.5
                        e'4. \startTextSpan
                        f'4.
                        \break
                        g'4.
                        a'4.
                        b'4. \stopTextSpan ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    60
                                }
                            }
                        c''4.
                    }
                >>

            Results in parenthesized abbreviation after line break.
            (This is default behavior.)

        ..  container:: example

            With left broken text set explicitly:

            ::

                >>> staff = Staff("c'4. d' e' f' g' a' b' c''")
                >>> attach(TimeSignature((3, 8)), staff)
                >>> score = Score([staff])
                >>> command = indicatortools.LilyPondCommand('break', 'after')
                >>> attach(command, staff[3])

            ::

                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> attach(tempo, staff[2], is_annotation=True)
                >>> tempo = Tempo(Duration(1, 4), 60)
                >>> attach(tempo, staff[6], is_annotation=True)

            ::

                >>> ritardando = indicatortools.Ritardando()
                >>> attach(ritardando, staff[2], is_annotation=True)

            ::

                >>> null_markup = Markup.null(direction=None)
                >>> tempo_spanner = spannertools.TempoSpanner(
                ...     left_broken_text=null_markup,
                ...     )
                >>> attach(tempo_spanner, staff[:])

            ::

                >>> override(score).text_script.staff_padding = 2.25
                >>> override(score).text_spanner.staff_padding = 3

            ::

                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score \with {
                    \override TextScript.staff-padding = #2.25
                    \override TextSpanner.staff-padding = #3
                } <<
                    \new Staff {
                        \time 3/8
                        c'4.
                        d'4.
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.padding = -2
                        \once \override TextSpanner.bound-details.left-broken.text = \markup {
                            \null
                            }
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    90
                                }
                            \hspace
                                #1.25
                            }
                        \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right-broken.text = ##f
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 2
                        \once \override TextSpanner.bound-details.right.text = ##f
                        \once \override TextSpanner.dash-fraction = 0.25
                        \once \override TextSpanner.dash-period = 1.5
                        e'4. \startTextSpan
                        f'4.
                        \break
                        g'4.
                        a'4.
                        b'4. \stopTextSpan ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    60
                                }
                            }
                        c''4.
                    }
                >>

            Results in null left broken text.

        Returns markup or none.
        '''
        return self._left_broken_text

    @property
    def start_with_parenthesized_tempo(self):
        r'''Is true when spanner should start with parenthesized tempo.

        ..  container:: example

            Does not start with parenthesized tempo:

            ::

                >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
                >>> attach(TimeSignature((3, 8)), staff)
                >>> score = Score([staff])

            ::

                >>> tempo = Tempo(Duration(1, 4), 60)
                >>> attach(tempo, staff[0], is_annotation=True)
                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> attach(tempo, staff[2], is_annotation=True)
                >>> tempo = Tempo(Duration(1, 4), 120)
                >>> attach(tempo, staff[5], is_annotation=True)

            ::

                >>> accelerando = indicatortools.Accelerando()
                >>> attach(accelerando, staff[3], is_annotation=True)

            ::

                >>> tempo_spanner = spannertools.TempoSpanner(
                ...     start_with_parenthesized_tempo=False,
                ...     )
                >>> attach(tempo_spanner, staff[:])

            ::

                >>> override(score).text_script.staff_padding = 2.25
                >>> override(score).text_spanner.staff_padding = 3

            ::

                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score \with {
                    \override TextScript.staff-padding = #2.25
                    \override TextSpanner.staff-padding = #3
                } <<
                    \new Staff {
                        \time 3/8
                        c'8. ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    60
                                }
                            }
                        d'8.
                        e'4. ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    90
                                }
                            }
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.padding = -2
                        \once \override TextSpanner.bound-details.left-broken.text = \markup {
                            \override
                                #'(padding . 0.45)
                                \parenthesize
                                    \large
                                        \upright
                                            accel.
                            \hspace
                                #0.75
                            }
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \large
                                \upright
                                    accel.
                            \hspace
                                #0.75
                            }
                        \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right-broken.text = ##f
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 2
                        \once \override TextSpanner.bound-details.right.text = ##f
                        \once \override TextSpanner.dash-fraction = 0.25
                        \once \override TextSpanner.dash-period = 1.5
                        g'8. \startTextSpan
                        f'8.
                        ef'4. \stopTextSpan ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    120
                                }
                            }
                    }
                >>

        ..  container:: example

            Starts with parenthesized tempo:

            ::

                >>> staff = Staff("c'8. d' e'4. g'8. f' ef'4.")
                >>> attach(TimeSignature((3, 8)), staff)
                >>> score = Score([staff])

            ::

                >>> tempo = Tempo(Duration(1, 4), 60)
                >>> attach(tempo, staff[0], is_annotation=True)
                >>> tempo = Tempo(Duration(1, 4), 90)
                >>> attach(tempo, staff[2], is_annotation=True)
                >>> tempo = Tempo(Duration(1, 4), 120)
                >>> attach(tempo, staff[5], is_annotation=True)

            ::

                >>> accelerando = indicatortools.Accelerando()
                >>> attach(accelerando, staff[3], is_annotation=True)

            ::

                >>> tempo_spanner = spannertools.TempoSpanner(
                ...     start_with_parenthesized_tempo=True,
                ...     )
                >>> attach(tempo_spanner, staff[:])

            ::

                >>> override(score).text_script.staff_padding = 2.25
                >>> override(score).text_spanner.staff_padding = 3

            ::

                >>> show(score) # doctest: +SKIP

            ..  doctest::

                >>> f(score)
                \new Score \with {
                    \override TextScript.staff-padding = #2.25
                    \override TextSpanner.staff-padding = #3
                } <<
                    \new Staff {
                        \time 3/8
                        c'8. ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    60
                                }
                            }
                        d'8.
                        e'4. ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    90
                                }
                            }
                        \once \override TextSpanner.arrow-width = 0.25
                        \once \override TextSpanner.bound-details.left-broken.padding = -2
                        \once \override TextSpanner.bound-details.left-broken.text = \markup {
                            \override
                                #'(padding . 0.45)
                                \parenthesize
                                    \large
                                        \upright
                                            accel.
                            \hspace
                                #0.75
                            }
                        \once \override TextSpanner.bound-details.left.stencil-align-dir-y = -0.5
                        \once \override TextSpanner.bound-details.left.text = \markup {
                            \override
                                #'(padding . 0.45)
                                \parenthesize
                                    \line
                                        {
                                            \fontsize
                                                #-6
                                                \general-align
                                                    #Y
                                                    #DOWN
                                                    \note-by-number
                                                        #2
                                                        #0
                                                        #1
                                            \upright
                                                {
                                                    =
                                                    90
                                                }
                                        }
                            \hspace
                                #0.75
                            }
                        \once \override TextSpanner.bound-details.right-broken.arrow = ##f
                        \once \override TextSpanner.bound-details.right-broken.padding = 0
                        \once \override TextSpanner.bound-details.right-broken.text = ##f
                        \once \override TextSpanner.bound-details.right.arrow = ##t
                        \once \override TextSpanner.bound-details.right.padding = 2
                        \once \override TextSpanner.bound-details.right.text = ##f
                        \once \override TextSpanner.dash-fraction = 0.25
                        \once \override TextSpanner.dash-period = 1.5
                        g'8. \startTextSpan
                        f'8.
                        ef'4. \stopTextSpan ^ \markup {
                            \fontsize
                                #-6
                                \general-align
                                    #Y
                                    #DOWN
                                    \note-by-number
                                        #2
                                        #0
                                        #1
                            \upright
                                {
                                    =
                                    120
                                }
                            }
                    }
                >>

        Defaults to false.

        Set to true, false or none.
        '''
        return self._start_with_parenthesized_tempo



# -*- coding: utf-8 -*-
import abjad
from abjad.tools.datastructuretools import TypedOrderedDict


def test_datastructuretools_TypedOrderedDict_01():
    r'''Implements __contains__().
    '''

    dictionary = TypedOrderedDict(item_class=abjad.Clef)
    dictionary['soprano'] = 'treble'
    dictionary['alto'] = 'alto'
    dictionary['tenor'] = 'tenor'
    dictionary['bass'] = 'bass'

    assert 'soprano' in dictionary
    assert 'treble' not in dictionary
    assert [_ for _ in dictionary] == ['soprano', 'alto', 'tenor', 'bass']


def test_datastructuretools_TypedOrderedDict_02():
    r'''Implements __delitem__().
    '''

    dictionary_1 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_1['soprano'] = 'treble'
    del(dictionary_1['soprano'])

    dictionary_2 = TypedOrderedDict(item_class=abjad.Clef)
    assert dictionary_1 == dictionary_2


def test_datastructuretools_TypedOrderedDict_03():
    r'''Implements __eq__().
    '''

    dictionary_1 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_1['soprano'] = 'treble'

    dictionary_2 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_2['soprano'] = 'treble'

    dictionary_3 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_3['bass'] = 'bass'

    assert dictionary_1 == dictionary_1
    assert dictionary_1 == dictionary_2
    assert not dictionary_1 == dictionary_3
    assert dictionary_2 == dictionary_1
    assert dictionary_2 == dictionary_2
    assert not dictionary_2 == dictionary_3
    assert not dictionary_3 == dictionary_1
    assert not dictionary_3 == dictionary_2
    assert dictionary_3 == dictionary_3


def test_datastructuretools_TypedOrderedDict_04():
    r'''Implements __format__().
    '''

    dictionary_1 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_1['soprano'] = 'treble'
    dictionary_1['alto'] = 'alto'
    dictionary_1['tenor'] = 'tenor'
    dictionary_1['bass'] = 'bass'

    assert format(dictionary_1) == abjad.stringtools.normalize(
        r'''
        abjad.datastructuretools.TypedOrderedDict(
            [
                (
                    'soprano',
                    indicatortools.Clef(
                        name='treble',
                        ),
                    ),
                (
                    'alto',
                    indicatortools.Clef(
                        name='alto',
                        ),
                    ),
                (
                    'tenor',
                    indicatortools.Clef(
                        name='tenor',
                        ),
                    ),
                (
                    'bass',
                    indicatortools.Clef(
                        name='bass',
                        ),
                    ),
                ],
            item_class=indicatortools.Clef,
            )
        '''
        )

    globs = abjad.__dict__.copy()
    globs['abjad'] = abjad
    dictionary_2 = eval(format(dictionary_1), globs)
    assert dictionary_1 == dictionary_2


def test_datastructuretools_TypedOrderedDict_05():
    r'''Initializes from dictionary items.
    '''

    items = [
        ('soprano', abjad.Clef('treble')),
        ('alto', abjad.Clef('alto')),
        ('tenor', abjad.Clef('tenor')),
        ('bass', abjad.Clef('bass')),
        ]
    dictionary_1 = TypedOrderedDict(
        item_class=abjad.Clef,
        items=items,
        )

    dictionary_2 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_2['soprano'] = 'treble'
    dictionary_2['alto'] = 'alto'
    dictionary_2['tenor'] = 'tenor'
    dictionary_2['bass'] = 'bass'

    assert dictionary_1 == dictionary_2


def test_datastructuretools_TypedOrderedDict_06():
    r'''Implements __len__().
    '''

    dictionary = TypedOrderedDict(item_class=abjad.Clef)
    assert len(dictionary) == 0

    dictionary = TypedOrderedDict(item_class=abjad.Clef)
    dictionary['soprano'] = 'treble'
    assert len(dictionary) == 1


def test_datastructuretools_TypedOrderedDict_07():
    r'''Implements __ne__().
    '''

    dictionary_1 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_1['soprano'] = 'treble'

    dictionary_2 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_2['soprano'] = 'treble'

    dictionary_3 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_3['bass'] = 'bass'

    assert not dictionary_1 != dictionary_1
    assert not dictionary_1 != dictionary_2
    assert dictionary_1 != dictionary_3
    assert not dictionary_2 != dictionary_1
    assert not dictionary_2 != dictionary_2
    assert dictionary_2 != dictionary_3
    assert dictionary_3 != dictionary_1
    assert dictionary_3 != dictionary_2
    assert not dictionary_3 != dictionary_3


def test_datastructuretools_TypedOrderedDict_08():
    r'''Implements __reversed__().
    '''

    dictionary = TypedOrderedDict(item_class=abjad.Clef)
    dictionary['soprano'] = 'treble'
    dictionary['alto'] = 'alto'
    dictionary['tenor'] = 'tenor'
    dictionary['bass'] = 'bass'

    generator = reversed(dictionary)
    assert [_ for _ in generator] == ['bass', 'tenor', 'alto', 'soprano']


def test_datastructuretools_TypedOrderedDict_09():
    r'''Implements clear().
    '''

    dictionary_1 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_1['soprano'] = 'treble'
    dictionary_1['alto'] = 'alto'
    dictionary_1['tenor'] = 'tenor'
    dictionary_1['bass'] = 'bass'
    dictionary_1.clear()

    dictionary_2 = TypedOrderedDict(item_class=abjad.Clef)
    assert dictionary_1 == dictionary_2


def test_datastructuretools_TypedOrderedDict_10():
    r'''Implements copy().
    '''

    dictionary_1 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_1['soprano'] = 'treble'
    dictionary_1['alto'] = 'alto'
    dictionary_1['tenor'] = 'tenor'
    dictionary_1['bass'] = 'bass'

    dictionary_2 = dictionary_1.copy()
    assert dictionary_1 == dictionary_2


def test_datastructuretools_TypedOrderedDict_11():
    r'''Implements get().
    '''

    dictionary = TypedOrderedDict(item_class=abjad.Clef)
    dictionary['soprano'] = 'treble'
    dictionary['alto'] = 'alto'
    dictionary['tenor'] = 'tenor'
    dictionary['bass'] = 'bass'

    assert dictionary.get('soprano') == abjad.Clef('treble')
    assert dictionary.get('foo') is None
    assert dictionary.get('foo', 'bar') == 'bar'


def test_datastructuretools_TypedOrderedDict_12():
    r'''Implements has_key().
    '''

    dictionary = TypedOrderedDict(item_class=abjad.Clef)
    dictionary['soprano'] = 'treble'
    dictionary['alto'] = 'alto'
    dictionary['tenor'] = 'tenor'
    dictionary['bass'] = 'bass'

    assert dictionary.has_key('soprano')
    assert not dictionary.has_key('treble')
    assert not dictionary.has_key('foo')


def test_datastructuretools_TypedOrderedDict_13():
    r'''Implements items().
    '''

    dictionary = TypedOrderedDict(item_class=abjad.Clef)
    dictionary['soprano'] = 'treble'
    dictionary['alto'] = 'alto'
    dictionary['tenor'] = 'tenor'
    dictionary['bass'] = 'bass'

    assert list(dictionary.items()) == [
        ('soprano', abjad.Clef('treble')),
        ('alto', abjad.Clef('alto')),
        ('tenor', abjad.Clef('tenor')),
        ('bass', abjad.Clef('bass')),
        ]


def test_datastructuretools_TypedOrderedDict_14():
    r'''Implements keys().
    '''

    dictionary = TypedOrderedDict(item_class=abjad.Clef)
    dictionary['soprano'] = 'treble'
    dictionary['alto'] = 'alto'
    dictionary['tenor'] = 'tenor'
    dictionary['bass'] = 'bass'

    assert list(dictionary.keys()) == ['soprano', 'alto', 'tenor', 'bass']


def test_datastructuretools_TypedOrderedDict_15():
    r'''Implements pop().
    '''

    dictionary_1 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_1['soprano'] = 'treble'
    dictionary_1['alto'] = 'alto'
    dictionary_1['tenor'] = 'tenor'
    dictionary_1['bass'] = 'bass'

    dictionary_2 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_2['alto'] = 'alto'
    dictionary_2['tenor'] = 'tenor'
    dictionary_2['bass'] = 'bass'

    assert dictionary_1.pop('soprano') == abjad.Clef('treble')
    assert dictionary_1 == dictionary_2


#def test_datastructuretools_TypedOrderedDict_16():
#    r'''Implements popitem().
#    '''
#
#    dictionary_1 = TypedOrderedDict(item_class=Clef)
#    dictionary_1['soprano'] = 'treble'
#    dictionary_1['alto'] = 'alto'
#    dictionary_1['tenor'] = 'tenor'
#    dictionary_1['bass'] = 'bass'
#
#    dictionary_2 = TypedOrderedDict(item_class=Clef)
#    dictionary_2['alto'] = 'alto'
#    dictionary_2['tenor'] = 'tenor'
#    dictionary_2['bass'] = 'bass'
#
#    item = dictionary_1.popitem('soprano')
#    assert item == ('soprano', Clef('treble'))
#    assert dictionary_1 == dictionary_2


def test_datastructuretools_TypedOrderedDict_17():
    r'''Implements update().
    '''

    dictionary_1 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_1['soprano'] = 'treble'
    dictionary_1['alto'] = 'alto'

    dictionary_2 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_2['tenor'] = 'tenor'
    dictionary_2['bass'] = 'bass'

    dictionary_3 = TypedOrderedDict(item_class=abjad.Clef)
    dictionary_3['soprano'] = 'treble'
    dictionary_3['alto'] = 'alto'
    dictionary_3['tenor'] = 'tenor'
    dictionary_3['bass'] = 'bass'

    dictionary_1.update(dictionary_2)
    assert dictionary_1 == dictionary_3



# -*- coding: utf-8 -*-


def edit_bass_voice(score, durated_reservoir):
    r'''Edits bass voice.
    '''

    voice = score['Bass Voice']

    voice[-3:] = '<e, e>\maxima <d, d>\longa <c, c>\maxima <b,>\longa <a,>\maxima r4 r2.'



# -*- coding: utf-8 -*-
import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class ContextManager(AbjadObject):
    r'''An abstract context manager class.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INTIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        raise NotImplementedError

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __enter__(self):
        r'''Enters context manager.
        '''
        raise NotImplementedError

    @abc.abstractmethod
    def __exit__(self, exc_type, exc_value, traceback):
        r'''Exits context manager.
        '''
        raise NotImplementedError



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_TieSpecifier_01():

    divisions = [(3, 8), (5, 16), (1, 4), (5, 16)]
    rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
        burnish_specifier=rhythmmakertools.BurnishSpecifier(
            left_classes=[Rest, Note],
            left_counts=[1],
            ),
        extra_counts_per_division=[0, 1, 1],
        talea=rhythmmakertools.Talea(
            counts=[1, 2, 3],
            denominator=16,
            ),
        tie_specifier=rhythmmakertools.TieSpecifier(
            tie_across_divisions=True,
            ),
        )
    selections = rhythm_maker(divisions)
    staff = Staff(selections)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                r16
                c'8 [
                c'8. ~ ]
            }
            \tweak text #tuplet-number::calc-fraction-text
            \times 5/6 {
                c'16 [
                c'8
                c'8. ]
            }
            \times 4/5 {
                r16
                c'8 [
                c'8 ~ ]
            }
            {
                c'16 [
                c'16
                c'8
                c'16 ]
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools.TreeNode import TreeNode


class ReSTHeading(TreeNode):
    r'''A ReST heading.

    ::

        >>> heading = documentationtools.ReSTHeading(
        ...     level=2, text='Section A')
        >>> heading
        ReSTHeading(
            level=2,
            text='Section A'
            )

    ::

        >>> print(heading.rest_format)
        Section A
        =========

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(self, level=0, name=None, text='foo'):
        TreeNode.__init__(self, name=name)
        self.level = level
        self.text = text

    ### PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        result = []
        underline, overline = self.heading_characters[self.level]
        if overline:
            result.append(overline * len(self.text))
        result.append(self.text)
        result.append(underline * len(self.text))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def heading_characters(self):
        r'''Heading characters of ReST heading.
        '''
        return (
            ('#', '#'),
            ('*', '*'),
            ('=', None),
            ('-', None),
            ('^', None),
            ('"', None),
        )

    @property
    def level(self):
        r'''Gets and sets level of ReST heading.
        '''
        return self._level

    @level.setter
    def level(self, arg):
        assert isinstance(arg, int) and \
            0 <= arg < len(self.heading_characters)
        self._level = arg

    @property
    def rest_format(self):
        r'''ReST format of ReST heading.

        Returns string.
        '''
        return '\n'.join(self._rest_format_contributions)

    @property
    def text(self):
        r'''Gets and sets text of ReST heading.

        Returns string.
        '''
        return self._text

    @text.setter
    def text(self, arg):
        assert isinstance(arg, str)
        arg = arg.strip()
        assert len(arg)
        self._text = arg



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_InspectionAgent_get_timespan_01():

    voice = Voice("c'8 d'8 e'8 f'8")
    for i, x in enumerate(voice):
        assert inspect_(x).get_timespan().start_offset == i * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    for i, x in enumerate(staff):
        assert inspect_(x).get_timespan().start_offset == i * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_03():

    staff = Staff("c'8 d'8 e'8 f'8")
    staff[-1] = Rest((1, 8))
    for i, x in enumerate(staff):
        assert inspect_(x).get_timespan().start_offset == i * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_04():

    staff = Staff("c'8 d'8 e'8 f'8")
    staff[10:10] = [Rest((1, 8))]
    for i, x in enumerate(staff):
        assert inspect_(x).get_timespan().start_offset == i * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_05():

    staff = Staff("c'8 d'8 e'8 f'8")
    staff[10:12] = [Rest((1, 8))]
    for i, x in enumerate(staff):
        assert inspect_(x).get_timespan().start_offset == i * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_06():
    r'''Offset works with voices.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'8 d'8 e'8 f'8")
    voice_1.name = voice_2.name = 'voice'
    container = Container([voice_1, voice_2])
    leaves = select(container).by_leaf()
    for i, x in enumerate(leaves):
        assert inspect_(x).get_timespan().start_offset == i * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_07():

    tuplet = scoretools.FixedDurationTuplet(Duration(1, 4), "c'8 c'8 c'8")
    for i, x in enumerate(tuplet):
        assert inspect_(x).get_timespan().start_offset == i * Offset(1, 12)


def test_agenttools_InspectionAgent_get_timespan_08():

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(1, 4), "c'8 c'8 c'8")
    voice = Voice([Note(0, (1, 8)), tuplet_1, Note(0, (1, 8))])
    offset = 0
    durations = [(1, 8), (1, 12), (1, 12), (1, 12), (1, 8)]
    leaves = select(voice).by_leaf()
    for leaf, duration in zip(leaves, durations):
        assert inspect_(leaf).get_timespan().start_offset == offset
        offset += Offset(*duration)


def test_agenttools_InspectionAgent_get_timespan_09():
    r'''Offset works on nested tuplets.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(1, 4), "c'8 c'8 c'8")
    tuplet = scoretools.FixedDurationTuplet(
        Duration(2, 4), [Note("c'4"), tuplet_1, Note("c'4")])
    offset = 0
    durations = [(1, 6), (1, 18), (1, 18), (1, 18), (1, 6)]
    leaves = select(tuplet).by_leaf()
    for leaf, duration in zip(leaves, durations):
        assert inspect_(leaf).get_timespan().start_offset == offset
        offset += Offset(*duration)


def test_agenttools_InspectionAgent_get_timespan_10():
    r'''Offset works with simultaneous structures.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'8 d'8 e'8 f'8")
    staff = Staff([voice_1, voice_2])
    staff.is_simultaneous = True
    for i, leaf in enumerate(voice_1):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8)
    for i, leaf in enumerate(voice_2):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_11():
    r'''Offset on leaves works in nested contexts.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    staff = Staff([Note(0, (1, 8)), voice, Note(0, (1, 8))])
    leaves = select(staff).by_leaf()
    for i, leaf in enumerate(leaves):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8)
    leaves = select(voice).by_leaf()
    for i, leaf in enumerate(leaves):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8) + Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_12():
    r'''Offset on leaves works in sequential contexts.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'8 d'8 e'8 f'8")
    staff = Staff([voice_1, voice_2])
    for i, leaf in enumerate(voice_1):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8)
    for i, leaf in enumerate(voice_2):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8) + Offset(1, 2)


def test_agenttools_InspectionAgent_get_timespan_13():
    r'''Offset on leaves works in nested simultaneous contexts.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'8 d'8 e'8 f'8")
    staff = Staff([voice_1, voice_2])
    staff.is_simultaneous = True
    for i, leaf in enumerate(voice_1):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8)
    for i, leaf in enumerate(voice_2):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_14():
    r'''Offset on leaves works in nested simultaneous and sequential contexts.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'8 d'8 e'8 f'8")
    voice_3 = Voice("c'8 d'8 e'8 f'8")
    staff = Staff([Container([voice_1, voice_2]), voice_3])
    staff[0].is_simultaneous = True
    for i, leaf in enumerate(voice_3):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8) + Offset(1, 2)


def test_agenttools_InspectionAgent_get_timespan_15():
    r'''Offset on leaves works in nested simultaneous and sequential contexts.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'8 d'8 e'8 f'8")
    voice_3 = Voice("c'8 d'8 e'8 f'8")
    staff = Staff([voice_3, Container([voice_1, voice_2])])
    staff[1].is_simultaneous = True
    for i, leaf in enumerate(voice_1):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8) + Offset(1, 2)
    for i, leaf in enumerate(voice_2):
        start_offset = inspect_(leaf).get_timespan().start_offset
        assert start_offset == i * Offset(1, 8) + Offset(1, 2)


def test_agenttools_InspectionAgent_get_timespan_16():
    r'''Offsets works on sequential voices.
    '''

    staff = Staff([Voice("c'8 d'8 e'8 f'8"), Voice("c'8 d'8 e'8 f'8")])
    staff[0].name = staff[1].name = 'voice'
    for i, voice in enumerate(staff):
        start_offset = inspect_(voice).get_timespan().start_offset
        assert start_offset == i * Offset(4, 8)


def test_agenttools_InspectionAgent_get_timespan_17():
    r'''Prolated offset does NOT go across sequential staves.
    '''

    container = Container([Staff("c'8 d'8 e'8 f'8"), Staff("c'8 d'8 e'8 f'8")])
    container[0].name = container[1].name = 'staff'
    start_offset = inspect_(container[0]).get_timespan().start_offset
    assert start_offset == Offset(0)
    start_offset = inspect_(container[1]).get_timespan().start_offset
    assert start_offset == Offset(1, 2)


def test_agenttools_InspectionAgent_get_timespan_18():
    r'''Offsets works with nested voices.
    '''

    staff = Staff([Voice("c'8 d'8 e'8 f'8"), Voice("c'8 d'8 e'8 f'8")])
    for i, voice in enumerate(staff):
        start_offset = inspect_(voice).get_timespan().start_offset
        assert start_offset == i * Offset(4, 8)


def test_agenttools_InspectionAgent_get_timespan_19():
    r'''Offsets works on sequential tuplets.
    '''

    voice = Voice(3 * Tuplet(Multiplier(2, 3), "c'8 d'8 e'8"))
    assert inspect_(voice[0]).get_timespan().start_offset == 0 * Offset(1, 4)
    assert inspect_(voice[1]).get_timespan().start_offset == 1 * Offset(1, 4)
    assert inspect_(voice[2]).get_timespan().start_offset == 2 * Offset(1, 4)


def test_agenttools_InspectionAgent_get_timespan_20():
    r'''Offsets work on tuplets between notes.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(1, 4), "c'8 c'8 c'8")
    voice = Voice([Note(0, (1, 8)), tuplet_1, Note(0, (1, 8))])
    assert inspect_(voice[0]).get_timespan().start_offset == 0 * Offset(1, 8)
    assert inspect_(voice[1]).get_timespan().start_offset == 1 * Offset(1, 8)
    assert inspect_(voice[2]).get_timespan().start_offset == 3 * Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_21():
    r'''Offsets work on nested tuplets.
    '''

    tuplet_1 = scoretools.FixedDurationTuplet(Duration(1, 4), "c'8 d'8 e'8 f'8")
    contents = [Note("c'4"), tuplet_1, Note("c'4")]
    tuplet = scoretools.FixedDurationTuplet(Duration(2, 4), contents)
    assert inspect_(tuplet[0]).get_timespan().start_offset == 0 * Offset(1, 6)
    assert inspect_(tuplet[1]).get_timespan().start_offset == 1 * Offset(1, 6)
    assert inspect_(tuplet[2]).get_timespan().start_offset == 2 * Offset(1, 6)


def test_agenttools_InspectionAgent_get_timespan_22():
    r'''Offsets work on nested contexts.
    '''

    inner_voice = Voice("c'8 d'8 e'8 f'8")
    outer_voice = Voice([Note(0, (1, 8)), inner_voice])
    inner_voice.name = outer_voice.name = 'voice'
    staff = Staff([Note(1, (1, 8)), outer_voice])
    assert inspect_(inner_voice).get_timespan().start_offset == Offset(2, 8)
    assert inspect_(outer_voice).get_timespan().start_offset == Offset(1, 8)


def test_agenttools_InspectionAgent_get_timespan_23():
    r'''Offsets work on nested simultaneous contexts.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'8 d'8 e'8 f'8")
    staff = Staff([voice_1, voice_2])
    staff.is_simultaneous = True
    assert inspect_(staff[0]).get_timespan().start_offset == 0
    assert inspect_(staff[1]).get_timespan().start_offset == 0


def test_agenttools_InspectionAgent_get_timespan_24():
    r'''Offsets works in nested simultaneous and sequential contexts.
    '''

    voice_1 = Voice("c'8 d'8 e'8 f'8")
    voice_2 = Voice("c'8 d'8 e'8 f'8")
    voice_1b = Voice("c'8 d'8 e'8 f'8")
    voice_2b = Voice("c'8 d'8 e'8 f'8")
    voice_1.name = voice_1b.name = 'voiceOne'
    staff_1 = Staff([voice_1, voice_1b])
    staff_2 = Staff([voice_2, voice_2b])
    gs = StaffGroup([staff_1, staff_2])
    assert inspect_(voice_1).get_timespan().start_offset == 0
    assert inspect_(voice_2).get_timespan().start_offset == 0
    assert inspect_(voice_1b).get_timespan().start_offset == Offset(4, 8)
    assert inspect_(voice_2b).get_timespan().start_offset == Offset(4, 8)


def test_agenttools_InspectionAgent_get_timespan_25():
    r'''Offset seconds can not calculate without excplit tempo indication.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")

    statement = 'inspect_(staff[0]).get_timespan(in_seconds=True).start_offset'
    assert pytest.raises(Exception, statement)


def test_agenttools_InspectionAgent_get_timespan_26():
    r'''Offset seconds work with explicit tempo indication.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    tempo = Tempo(Duration(1, 8), 48)
    attach(tempo, staff, scope=Staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \tempo 8=48
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )

    start_offset = inspect_(staff[0]).get_timespan(in_seconds=True).start_offset
    assert start_offset == Offset(0)
    start_offset = inspect_(staff[1]).get_timespan(in_seconds=True).start_offset
    assert start_offset == Offset(5, 4)



# -*- coding: utf-8 -*-
import os
import platform
from abjad.tools import commandlinetools
from abjad.tools import systemtools
from base import ScorePackageScriptTestCase
try:
    from unittest import mock
except ImportError:
    import mock


class Test(ScorePackageScriptTestCase):

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_all(self, open_file_mock):
        expected_files = [
            'test_score/test_score/build/.gitignore',
            'test_score/test_score/build/assets/.gitignore',
            'test_score/test_score/build/assets/instrumentation.tex',
            'test_score/test_score/build/assets/performance-notes.tex',
            'test_score/test_score/build/letter-portrait/back-cover.pdf',
            'test_score/test_score/build/letter-portrait/back-cover.tex',
            'test_score/test_score/build/letter-portrait/front-cover.pdf',
            'test_score/test_score/build/letter-portrait/front-cover.tex',
            'test_score/test_score/build/letter-portrait/music.ly',
            'test_score/test_score/build/letter-portrait/music.pdf',
            'test_score/test_score/build/letter-portrait/parts.ly',
            'test_score/test_score/build/letter-portrait/preface.pdf',
            'test_score/test_score/build/letter-portrait/preface.tex',
            'test_score/test_score/build/letter-portrait/score.pdf',
            'test_score/test_score/build/letter-portrait/score.tex',
            'test_score/test_score/build/parts.ily',
            'test_score/test_score/build/segments.ily',
            'test_score/test_score/build/segments/.gitignore',
            'test_score/test_score/build/segments/test-segment.ily',
            ]
        if platform.system().lower() == 'windows':
            expected_files = [_.replace('/', os.path.sep) for _ in expected_files]
        self.create_score()
        self.create_segment('test_segment')
        self.illustrate_segments()
        self.collect_segments()
        self.create_build_target()
        script = commandlinetools.ManageBuildTargetScript()
        command = ['--render', 'letter-portrait']
        #with systemtools.RedirectedStreams(stdout=self.string_io):
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        self.compare_path_contents(self.build_path, expected_files)
        assert open_file_mock.called

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_back_cover(self, open_file_mock):
        expected_files = [
            'test_score/test_score/build/letter-portrait/back-cover.pdf',
            'test_score/test_score/build/letter-portrait/back-cover.tex',
            'test_score/test_score/build/letter-portrait/front-cover.tex',
            'test_score/test_score/build/letter-portrait/music.ly',
            'test_score/test_score/build/letter-portrait/parts.ly',
            'test_score/test_score/build/letter-portrait/preface.tex',
            'test_score/test_score/build/letter-portrait/score.tex',
            ]
        if platform.system().lower() == 'windows':
            expected_files = [_.replace('/', os.path.sep) for _ in expected_files]
        self.create_score()
        self.create_segment('test_segment')
        self.illustrate_segments()
        self.collect_segments()
        target_path = self.create_build_target()
        script = commandlinetools.ManageBuildTargetScript()
        command = [
            '--render', 'letter-portrait',
            '--back-cover',
            ]
        #with systemtools.RedirectedStreams(stdout=self.string_io):
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        self.compare_path_contents(target_path, expected_files)
        assert open_file_mock.called

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_front_cover(self, open_file_mock):
        expected_files = [
            'test_score/test_score/build/letter-portrait/back-cover.tex',
            'test_score/test_score/build/letter-portrait/front-cover.pdf',
            'test_score/test_score/build/letter-portrait/front-cover.tex',
            'test_score/test_score/build/letter-portrait/music.ly',
            'test_score/test_score/build/letter-portrait/parts.ly',
            'test_score/test_score/build/letter-portrait/preface.tex',
            'test_score/test_score/build/letter-portrait/score.tex',
            ]
        if platform.system().lower() == 'windows':
            expected_files = [_.replace('/', os.path.sep) for _ in expected_files]
        self.create_score()
        self.create_segment('test_segment')
        self.illustrate_segments()
        self.collect_segments()
        target_path = self.create_build_target()
        script = commandlinetools.ManageBuildTargetScript()
        command = [
            '--render', 'letter-portrait',
            '--front-cover',
            ]
        #with systemtools.RedirectedStreams(stdout=self.string_io):
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        self.compare_path_contents(target_path, expected_files)
        assert open_file_mock.called

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_music(self, open_file_mock):
        expected_files = [
            'test_score/test_score/build/letter-portrait/back-cover.tex',
            'test_score/test_score/build/letter-portrait/front-cover.tex',
            'test_score/test_score/build/letter-portrait/music.ly',
            'test_score/test_score/build/letter-portrait/music.pdf',
            'test_score/test_score/build/letter-portrait/parts.ly',
            'test_score/test_score/build/letter-portrait/preface.tex',
            'test_score/test_score/build/letter-portrait/score.tex',
            ]
        if platform.system().lower() == 'windows':
            expected_files = [_.replace('/', os.path.sep) for _ in expected_files]
        self.create_score()
        self.create_segment('test_segment')
        self.illustrate_segments()
        self.collect_segments()
        target_path = self.create_build_target()
        script = commandlinetools.ManageBuildTargetScript()
        command = [
            '--render', 'letter-portrait',
            '--music',
            ]
        #with systemtools.RedirectedStreams(stdout=self.string_io):
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        self.compare_path_contents(target_path, expected_files)
        assert open_file_mock.called

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_parts(self, open_file_mock):
        expected_files = [
            'test_score/test_score/build/letter-portrait/back-cover.tex',
            'test_score/test_score/build/letter-portrait/front-cover.tex',
            'test_score/test_score/build/letter-portrait/music.ly',
            'test_score/test_score/build/letter-portrait/parts-cello.pdf',
            'test_score/test_score/build/letter-portrait/parts-viola.pdf',
            'test_score/test_score/build/letter-portrait/parts-violin-i.pdf',
            'test_score/test_score/build/letter-portrait/parts-violin-ii.pdf',
            'test_score/test_score/build/letter-portrait/parts.ly',
            'test_score/test_score/build/letter-portrait/preface.tex',
            'test_score/test_score/build/letter-portrait/score.tex',
            ]
        if platform.system().lower() == 'windows':
            expected_files = [_.replace('/', os.path.sep) for _ in expected_files]
        self.create_score()
        self.install_fancy_segment_maker()
        self.create_segment('test_segment')
        self.illustrate_segments()
        self.collect_segments()
        target_path = self.create_build_target()
        script = commandlinetools.ManageBuildTargetScript()
        command = [
            '--render', 'letter-portrait',
            '--parts',
            ]
        #with systemtools.RedirectedStreams(stdout=self.string_io):
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        self.compare_path_contents(target_path, expected_files)
        assert open_file_mock.called

    @mock.patch('abjad.systemtools.IOManager.open_file')
    def test_success_preface(self, open_file_mock):
        expected_files = [
            'test_score/test_score/build/letter-portrait/back-cover.tex',
            'test_score/test_score/build/letter-portrait/front-cover.tex',
            'test_score/test_score/build/letter-portrait/music.ly',
            'test_score/test_score/build/letter-portrait/parts.ly',
            'test_score/test_score/build/letter-portrait/preface.pdf',
            'test_score/test_score/build/letter-portrait/preface.tex',
            'test_score/test_score/build/letter-portrait/score.tex',
            ]
        if platform.system().lower() == 'windows':
            expected_files = [_.replace('/', os.path.sep) for _ in expected_files]
        self.create_score()
        self.create_segment('test_segment')
        self.illustrate_segments()
        self.collect_segments()
        target_path = self.create_build_target()
        script = commandlinetools.ManageBuildTargetScript()
        command = [
            '--render', 'letter-portrait',
            '--preface',
            ]
        #with systemtools.RedirectedStreams(stdout=self.string_io):
        with systemtools.TemporaryDirectoryChange(str(self.score_path)):
            try:
                script(command)
            except SystemExit:
                raise RuntimeError('SystemExit')
        self.compare_path_contents(target_path, expected_files)
        assert open_file_mock.called



# -*- coding: utf-8 -*-
from __future__ import print_function
import abc
import collections
import importlib
import json
import os
import re
import shutil
import sys
import subprocess
import traceback
from abjad.tools import systemtools
from abjad.tools import stringtools
from abjad.tools.commandlinetools.CommandlineScript import CommandlineScript
try:
    import pathlib
except ImportError:
    import pathlib2 as pathlib


class ScorePackageScript(CommandlineScript):
    '''
    Abstract base class for score-package scripts.
    '''

    ### CLASS VARIABLES ###

    _name_re = re.compile('^[a-z][a-z0-9_]*$')

    ### INITIALIZER ###

    def __init__(self):
        CommandlineScript.__init__(self)
        self._cwd = self._get_current_working_directory()
        self._score_package_path = None
        self._score_repository_path = None
        self._root_parent_path = None
        self._build_path = None
        self._segments_path = None
        self._materials_path = None

    ### PRIVATE METHODS ###

    def _call_subprocess(self, command):
        '''Trivial wrapper for mocking purposes.'''
        return subprocess.call(command, shell=True)

    def _collect_globbable_names(self, input_names):
        validated_names = []
        for input_name in input_names:
            if not self._name_is_valid_globbable(input_name):
                print('Cannot glob {!r}'.format(input_name))
                sys.exit(1)
            validated_names.append(input_name)
        return validated_names

    @classmethod
    def _copy_tree(cls, source_directory, target_directory, recurse=True):
        copied_paths = []
        source_paths = [_ for _ in source_directory.glob('*')]
        if not target_directory.exists():
            target_directory.mkdir(parents=True)
        for source_path in source_paths:
            if source_path.name == '__pycache__':
                continue
            elif source_path.suffix == '.pyc':
                continue
            source_name = source_path.relative_to(source_directory)
            target_path = target_directory.joinpath(source_name)
            if source_path.is_dir() and recurse:
                copied_paths.extend(
                    cls._copy_tree(source_path, target_path, recurse=True)
                    )
            elif source_path.is_file():
                shutil.copyfile(str(source_path), str(target_path))
            copied_paths.append(target_path)
        return copied_paths

    def _create_build_target(
        self,
        name,
        score_path,
        paper_size,
        orientation,
        force=False,
        ):
        from abjad import abjad_configuration
        target_path = self._name_to_score_subdirectory_path(
            name, 'build', score_path)
        if target_path.exists() and not force:
            print('    Path exists: {!s}'.format(
                target_path.relative_to(self._score_package_path.parent)))
            sys.exit(1)
        metadata = self._read_score_metadata_json(score_path)
        metadata['score_package_name'] = score_path.name
        if orientation == 'portrait':
            rotation = 0
            width, height = self.paper_sizes[paper_size]
        else:
            rotation = -90
            height, width = self.paper_sizes[paper_size]
        metadata['rotation'] = rotation
        metadata['orientation'] = orientation
        metadata['paper_size'] = paper_size
        metadata['height'] = height[0]
        metadata['width'] = width[0]
        metadata['unit'] = height[1]
        metadata['global_staff_size'] = 12
        metadata['uppercase_composer_name'] = metadata['composer_name'].upper()
        metadata['uppercase_title'] = metadata['title'].upper()
        metadata['lilypond_version'] = \
            abjad_configuration.get_lilypond_version_string()
        source_name = 'example_build_target'
        source_path = self._get_boilerplate_path().joinpath(source_name)
        suffixes = ('.py', '.tex', '.ly', '.ily')
        for path in self._copy_tree(source_path, target_path):
            if path.is_file() and path.suffix in suffixes:
                self._template_file(path, **metadata)
        return target_path

    @classmethod
    def _get_boilerplate_path(cls):
        import abjad
        return pathlib.Path(abjad.__path__[0]).joinpath('boilerplate')

    @classmethod
    def _get_current_working_directory(self):
        return pathlib.Path('.').absolute()

    def _import_all_materials(self, verbose=True):
        materials = collections.OrderedDict()
        for path in self._list_material_subpackages():
            name = path.name
            material = self._import_material(path, verbose=verbose)
            materials[name] = material
        return materials

    def _import_material(self, material_directory_path, verbose=True):
        material_import_path = self._path_to_packagesystem_path(
            material_directory_path)
        material_name = material_directory_path.name
        definition_import_path = material_import_path + '.definition'
        try:
            module = self._import_path(
                definition_import_path,
                self._score_repository_path,
                verbose=verbose,
                )
            material = getattr(module, material_name)
        except (ImportError, AttributeError):
            traceback.print_exc()
            sys.exit(1)
        return material

    def _import_path(self, path, score_root_path, verbose=True):
        if verbose:
            print('    Importing {!s}'.format(path))
        with systemtools.TemporaryDirectoryChange(str(score_root_path)):
            try:
                importlib.invalidate_caches()
            except:
                pass
            try:
                return importlib.import_module(path)
            except ImportError:
                traceback.print_exc()
                raise SystemExit(1)
            except Exception:
                traceback.print_exc()
                raise SystemExit(1)

    def _list_material_subpackages(self, score_path=None):
        materials_path = self._materials_path
        if score_path:
            score_path = self._path_to_score_package_path(score_path)
            materials_path = score_path.joinpath('materials')
        paths = [
            path for path in materials_path.glob('*')
            if path.is_dir() and path.joinpath('__init__.py').exists()
            ]
        return sorted(paths)

    def _list_segment_subpackages(self, score_path=None):
        segments_path = self._segments_path
        if score_path:
            score_path = self._path_to_score_package_path(score_path)
            segments_path = score_path.joinpath('segments')
        paths = [
            path for path in segments_path.glob('*')
            if path.is_dir() and path.joinpath('__init__.py').exists()
            ]
        return sorted(paths)

    @classmethod
    def _name_is_valid_globbable(cls, name):
        if '..' in name:
            return False
        elif '**' in name:
            return False
        elif '/' in name:
            return False
        return True

    def _name_to_score_subdirectory_path(self, name, section, score_path):
        score_path = self._path_to_score_package_path(score_path)
        name = stringtools.to_accent_free_snake_case(name)
        path = score_path.joinpath(section, name)
        return path

    def _path_to_packagesystem_path(self, path):
        score_package_path = self._path_to_score_package_path(path)
        relative_path = path.relative_to(score_package_path)
        parts = [score_package_path.name]
        parts.extend(relative_path.parts)
        return '.'.join(parts)

    def _path_to_score_package_path(self, path):
        if isinstance(path, str):
            # Is `path` an importable name? Use its module path.
            if self._name_re.match(path):
                try:
                    importlib.invalidate_caches()
                    module = importlib.import_module(path)
                    path = getattr(module, '__file__',  # A module.
                        getattr(module, '__path__'))  # A package.
                    if hasattr(path, '_path'):  # A local import.
                        path = path._path
                    if not isinstance(path, str):  # If it's a package...
                        path = path[0]  # Get the first path in the list.
                except:
                    traceback.print_exc()
            # Make sure to expand any home variables.
            path = pathlib.Path(os.path.expanduser(path))
        path = path.absolute()
        if not path.exists():
            print("Couldn locate or import score matching {!r}.".format(path))
            sys.exit(1)
        # Convert to directory.
        if path.is_file():
            path = path.parent
        # Check for parent package if not actually inside a package.
        # E.g.:
        #   - score_root
        #   - score_root/score/build/build_target
        #   - score_root/score/etc
        if not path.joinpath('__init__.py').exists():
            if path.joinpath(path.name, '__init__.py').exists():
                pass
            elif path.parent.joinpath('__init__.py').exists():
                path = path.parent
            elif path.parent.parent.joinpath('__init__.py').exists():
                path = path.parent.parent
        # Drill down as long as we're inside a Python package.
        while path.joinpath('__init__.py').exists():
            path = path.parent
        path = path.joinpath(path.name)
        # Make sure the directory even exists.
        if not path.exists():
            print('No score matching {!r} exists.'.format(path))
            sys.exit(1)
        # Check for mandatory files and subdirectories.
        if (
            not path.joinpath('__init__.py').exists() or
            not path.joinpath('materials').exists() or
            not path.joinpath('segments').exists() or
            not path.joinpath('build').exists() or
            not path.joinpath('tools').exists()
            ):
            print('Score directory {!r} is malformed.'.format(path))
            sys.exit(1)
        return path

    def _read_json(self, path, strict=False, verbose=True):
        if verbose:
            message = '    Reading {!s} ... '
            path_to_print = path.relative_to(self._score_package_path.parent)
            print(message.format(path_to_print), end='')
        if not path.exists():
            if verbose:
                print('JSON does not exist.')
            if strict:
                sys.exit(1)
            return {}
        try:
            with open(str(path), 'r') as file_pointer:
                argument = json.loads(file_pointer.read())
        except:
            if verbose:
                print('JSON is corrupted.')
            if strict:
                sys.exit(1)
            return {}
        if verbose:
            print('OK!')
        return argument

    def _read_score_metadata_json(self, score_path=None, verbose=True):
        if score_path:
            score_path = self._path_to_score_package_path(score_path)
        else:
            score_path = self._score_package_path
        metadata_path = score_path.joinpath('metadata.json')
        metadata = self._read_json(metadata_path, verbose=verbose)
        assert isinstance(metadata, dict)
        return metadata

    def _read_segments_list_json(self, score_path=None, verbose=True):
        if score_path:
            score_path = self._path_to_score_package_path(score_path)
        else:
            score_path = self._score_package_path
        listing_path = score_path.joinpath('segments', 'metadata.json')
        segment_paths = self._list_segment_subpackages(score_path)
        valid_segment_names = [_.name for _ in segment_paths]
        argument = self._read_json(listing_path, verbose=verbose)
        segment_names = argument.get('segments', [])
        if not isinstance(segment_names, list):
            if verbose:
                print('    Segments listing is malformed.')
            return []
        segment_names = [_ for _ in segment_names if _ in valid_segment_names]
        return segment_names

    def _report_time(self, timer, prefix='Runtime'):
        message = '        {}: {} {}'
        total_time = int(timer.elapsed_time)
        identifier = stringtools.pluralize('second', total_time)
        message = message.format(prefix, total_time, identifier)
        print(message)

    @abc.abstractmethod
    def _setup_argument_parser(self, parser):
        parser.add_argument(
            '-s', '--score-path',
            metavar='SCORE_PATH',
            default=os.path.curdir,
            )

    def _setup_paths(self, score_path):
        score_package_path = self._path_to_score_package_path(score_path)
        self._score_package_path = score_package_path
        self._score_repository_path = score_package_path.parent
        self._root_parent_path = self._score_repository_path.parent
        self._build_path = self._score_package_path.joinpath('build')
        self._distribution_path = self._score_package_path.joinpath('distribution')
        self._segments_path = self._score_package_path.joinpath('segments')
        self._materials_path = self._score_package_path.joinpath('materials')

    @classmethod
    def _template_file(cls, file_path, **keywords):
        file_path = str(file_path)
        with open(file_path, 'r') as file_pointer:
            template = file_pointer.read()
        try:
            completed_template = template.format(**keywords)
        except (IndexError, KeyError):
            #traceback.print_exc()
            lines = template.splitlines()
            for i, line in enumerate(lines):
                try:
                    lines[i] = line.format(**keywords)
                except (KeyError, IndexError, ValueError):
                    pass
            completed_template = '\n'.join(lines)
        with open(file_path, 'w') as file_pointer:
            file_pointer.write(completed_template)

    def _write_json(self, argument, path, verbose=True):
        if verbose:
            message = '    Writing {!s}'
            path_to_print = path.relative_to(self._score_repository_path)
            print(message.format(path_to_print))
        contents = json.dumps(
            argument,
            sort_keys=True,
            indent=4,
            separators=(',', ': '),
            )
        should_write = True
        if path.exists():
            with open(str(path), 'r') as file_pointer:
                if file_pointer.read() == contents:
                    should_write = False
        if should_write:
            with open(str(path), 'w') as file_pointer:
                file_pointer.write(contents)

    def _write_lilypond_ly(
        self,
        lilypond_file,
        output_directory_path,
        ):
        ly_path = output_directory_path.joinpath('illustration.ly')
        message = '    Writing {!s} ... '
        message = message.format(ly_path.relative_to(self._score_repository_path))
        print(message, end='')
        try:
            lilypond_format = format(lilypond_file)
        except:
            print('Failed!')
            traceback.print_exc()
            sys.exit(1)
        with open(str(ly_path), 'w') as file_pointer:
            file_pointer.write(lilypond_format)
        print('OK!')
        return ly_path

    def _write_lilypond_pdf(
        self,
        ly_path,
        output_directory_path,
        ):
        from abjad import abjad_configuration
        pdf_path = output_directory_path.joinpath('illustration.pdf')
        message = '    Writing {!s} ... '
        message = message.format(pdf_path.relative_to(self._score_repository_path))
        print(message, end='')
        command = '{} -dno-point-and-click -o {} {}'.format(
            abjad_configuration.get('lilypond_path', 'lilypond'),
            str(ly_path).replace('.ly', ''),
            str(ly_path),
            )
        with systemtools.Timer() as timer:
            with systemtools.TemporaryDirectoryChange(str(ly_path.parent)):
                exit_code = subprocess.call(command, shell=True)
        if exit_code:
            print('Failed!')
            sys.exit(1)
        print('OK!')
        self._report_time(timer, prefix='LilyPond runtime')

    def _write_score_metadata_json(self, score_path=None, verbose=True, **keywords):
        if score_path:
            score_path = self._path_to_score_package_path(score_path)
        else:
            score_path = self._score_package_path
        metadata_path = score_path.joinpath('metadata.json')
        self._write_json(keywords, metadata_path, verbose=verbose)

    def _write_segments_list_json(self, segment_names, score_path=None, verbose=True):
        if score_path:
            score_path = self._path_to_score_package_path(score_path)
        else:
            score_path = self._score_package_path
        listing_path = score_path.joinpath('segments', 'metadata.json')
        segment_paths = self._list_segment_subpackages(score_path)
        valid_segment_names = set(_.name for _ in segment_paths)
        segment_names = [_ for _ in segment_names if _ in valid_segment_names]
        argument = {'segments': segment_names}
        self._write_json(argument, listing_path, verbose=verbose)



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import systemtools
from abjad.tools.selectiontools.Selection import Selection


class VerticalMoment(Selection):
    r'''A selection of components happening at a single moment in musical time.

    ::

        >>> score = Score([])
        >>> staff_group = StaffGroup()
        >>> staff_group.context_name = 'PianoStaff'
        >>> staff_group.append(Staff("c'4 e'4 d'4 f'4"))
        >>> staff_group.append(Staff(r"""\clef "bass" g2 f2"""))
        >>> score.append(staff_group)

    ..  doctest::

        >>> print(format(score))
        \new Score <<
            \new PianoStaff <<
                \new Staff {
                    c'4
                    e'4
                    d'4
                    f'4
                }
                \new Staff {
                    \clef "bass"
                    g2
                    f2
                }
            >>
        >>

    ::

        >>> show(score) # doctest: +SKIP

    ::

        >>> for x in iterate(score).by_vertical_moment():
        ...     x
        ...
        VerticalMoment(0, <<2>>)
        VerticalMoment(1/4, <<2>>)
        VerticalMoment(1/2, <<2>>)
        VerticalMoment(3/4, <<2>>)

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_components',
        '_governors',
        '_offset',
        )

    ### INITIALIZER ###

    def __init__(self, music=None, offset=None):
        self._music = music
        if music is None:
            self._offset = offset
            self._components = ()
            self._governors = ()
            return
        governors, components = self._from_offset(music, offset)
        offset = durationtools.Offset(offset)
        assert isinstance(governors, tuple)
        assert isinstance(components, tuple)
        self._offset = offset
        self._governors = tuple(governors)
        components = list(components)
        components.sort(key=lambda x: x._get_parentage().score_index)
        self._components = tuple(components)

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a vertical moment with the same components as
        this vertical moment. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, VerticalMoment):
            if len(self) == len(argument):
                for c, d in zip(self.components, argument.components):
                    if c is not d:
                        return False
                else:
                    return True
        return False

    def __hash__(self):
        r'''Hases vertical moment.

        Returns integer.
        '''
        result = []
        result.append(str(self.offset))
        result.extend([str(id(x)) for x in self.governors])
        result = '+'.join(result)
        return hash(repr(result))

    def __len__(self):
        r'''Length of vertical moment.

        Defined equal to the number of components in vertical moment.

        Returns nonnegative integer.
        '''
        return len(self.components)

    def __ne__(self, argument):
        r'''Is true when `argument` does not equal this vertical moment. Otherwise
        false.

        Returns true or false.
        '''
        return not self == argument

    def __repr__(self):
        r'''Gets interpreter representation of vertical moment.

        Returns string.
        '''
        if not self.components:
            return '{}()'.format(type(self).__name__)
        result = '{}({}, <<{}>>)'
        result = result.format(
            type(self).__name__,
            str(self.offset),
            len(self.leaves),
            )
        return result

    ### PRIVATE METHODS ###

    @staticmethod
    def _find_index(container, offset):
        r'''Based off of Python's bisect.bisect() function.
        '''
        lo = 0
        hi = len(container)
        while lo < hi:
            mid = (lo + hi) // 2
            start_offset = container[mid]._get_timespan().start_offset
            stop_offset = container[mid]._get_timespan().stop_offset
            if start_offset <= offset < stop_offset:
                lo = mid + 1
            # if container[mid] is of nonzero duration
            elif start_offset < stop_offset:
                hi = mid
            # container[mid] is of zero duration so we skip it
            else:
                lo = mid + 1
        return lo - 1

    @staticmethod
    def _from_offset(argument, offset):
        from abjad.tools import scoretools
        from abjad.tools import selectiontools
        offset = durationtools.Offset(offset)
        governors = []
        prototype = (list, tuple, selectiontools.Selection)
        message = 'must be component or of Abjad components: {!r}.'
        message = message.format(argument)
        if isinstance(argument, scoretools.Component):
            governors.append(argument)
        elif isinstance(argument, prototype):
            for x in argument:
                if isinstance(x, scoretools.Component):
                    governors.append(x)
                else:
                    raise TypeError(message)
        else:
            raise TypeError(message)
        governors.sort(key=lambda x: x._get_parentage().score_index)
        governors = tuple(governors)
        components = []
        for governor in governors:
            components.extend(VerticalMoment._recurse(governor, offset))
        components.sort(key=lambda x: x._get_parentage().score_index)
        components = tuple(components)
        return governors, components

    def _get_format_specification(self):
        return systemtools.FormatSpecification(client=self)

    @staticmethod
    def _recurse(component, offset):
        result = []
        if component._get_timespan().start_offset <= \
            offset < component._get_timespan().stop_offset:
            result.append(component)
            if hasattr(component, '_music'):
                if component.is_simultaneous:
                    for x in component:
                        result.extend(VerticalMoment._recurse(x, offset))
                else:
                    child = component[
                        VerticalMoment._find_index(component, offset)]
                    result.extend(VerticalMoment._recurse(child, offset))
        return result

    ### PRIVATE PROPERTIES ###

    @property
    def _format_string(self):
        return ', '.join([str(x) for x in self.components])

    ### PUBLIC PROPERTIES ###

    @property
    def attack_count(self):
        r'''Positive integer number of pitch carriers
        starting at vertical moment.
        '''
        from abjad.tools import scoretools
        attack_carriers = []
        for leaf in self.start_leaves:
            if isinstance(leaf, (scoretools.Note, scoretools.Chord)):
                attack_carriers.append(leaf)
        return len(attack_carriers)

    @property
    def components(self):
        r'''Tuple of zero or more components
        happening at vertical moment.

        It is always the case that ``self.components =
        self.overlap_components + self.start_components``.
        '''
        return self._components

    @property
    def governors(self):
        r'''Tuple of one or more containers
        in which vertical moment is evaluated.
        '''
        return self._governors

    @property
    def leaves(self):
        r'''Tuple of zero or more leaves
        at vertical moment.
        '''
        from abjad.tools import scoretools
        result = []
        for component in self.components:
            if isinstance(component, scoretools.Leaf):
                result.append(component)
        result = tuple(result)
        return result

    @property
    def measures(self):
        r'''Tuplet of zero or more measures
        at vertical moment.
        '''
        from abjad.tools import scoretools
        result = []
        for component in self.components:
            if isinstance(component, scoretools.Measure):
                result.append(component)
        result = tuple(result)
        return result

    @property
    def music(self):
        r'''Gets music of vertical moment.

        Returns component or selection.
        '''
        return self._music

    @property
    def next_vertical_moment(self):
        r'''Reference to next vertical moment forward in time.
        '''
        from abjad.tools import scoretools
        candidate_shortest_leaf = self.leaves[0]
        for leaf in self.leaves[1:]:
            if leaf._get_timespan().stop_offset < \
                candidate_shortest_leaf._get_timespan().stop_offset:
                candidate_shortest_leaf = leaf
        next_leaf = candidate_shortest_leaf._get_in_my_logical_voice(
            1, prototype=scoretools.Leaf)
        next_vertical_moment = next_leaf._get_vertical_moment()
        return next_vertical_moment

    @property
    def notes(self):
        r'''Tuple of zero or more notes at vertical moment.
        '''
        from abjad.tools import scoretools
        result = []
        prototype = (scoretools.Note,)
        for component in self.components:
            if isinstance(component, prototype):
                result.append(component)
        result = tuple(result)
        return result

    @property
    def notes_and_chords(self):
        r'''Tuple of zero or more notes and chords at vertical moment.
        '''
        from abjad.tools import scoretools
        result = []
        prototype = (scoretools.Chord, scoretools.Note)
        for component in self.components:
            if isinstance(component, prototype):
                result.append(component)
        result = tuple(result)
        return result

    @property
    def offset(self):
        r'''Rational-valued score offset at which vertical moment is evaluated.
        '''
        return self._offset

    @property
    def overlap_components(self):
        r'''Tuple of components in vertical moment starting before vertical
        moment, ordered by score index.
        '''
        result = []
        for component in self.components:
            if component.start < self.offset:
                result.append(component)
        result = tuple(result)
        return result

    @property
    def overlap_leaves(self):
        r'''Tuple of leaves in vertical moment starting before vertical moment,
        ordered by score index.
        '''
        from abjad.tools import scoretools
        result = [x for x in self.overlap_components
            if isinstance(x, scoretools.Leaf)]
        result = tuple(result)
        return result

    @property
    def overlap_measures(self):
        r'''Tuple of measures in vertical moment starting before vertical
        moment, ordered by score index.
        '''
        from abjad.tools import scoretools
        result = [x for x in self.overlap_components
            if isinstance(x, scoretools.Measure)]
        result = tuple(result)
        return result

    @property
    def overlap_notes(self):
        r'''Tuple of notes in vertical moment starting before vertical moment,
        ordered by score index.
        '''
        from abjad.tools import scoretools
        result = [x for x in self.overlap_components
            if isinstance(x, scoretools.Note)]
        result = tuple(result)
        return result

    @property
    def previous_vertical_moment(self):
        r'''Reference to previous vertical moment backward in time.
        '''
        from abjad.tools import scoretools
        if self.offset == 0:
            raise IndexError
        most_recent_start_offset = durationtools.Offset(0)
        token_leaf = None
        for leaf in self.leaves:
            #print ''
            #print leaf
            leaf_start = leaf._get_timespan().start_offset
            if leaf_start < self.offset:
                #print 'found leaf starting before this moment ...'
                if most_recent_start_offset <= leaf_start:
                    most_recent_start_offset = leaf_start
                    token_leaf = leaf
            else:
                #print 'found leaf starting on this moment ...'
                try:
                    previous_leaf = leaf._get_in_my_logical_voice(
                        -1, prototype=scoretools.Leaf)
                    start = previous_leaf._get_timespan().start_offset
                    #print previous_leaf, start
                    if most_recent_start_offset <= start:
                        most_recent_start_offset = start
                        token_leaf = previous_leaf
                except IndexError:
                    pass
        #print 'token_leaf is %s ...' % token_leaf
        if token_leaf is None:
            token_leaf = leaf
            #print 'token_leaf is %s ...' % token_leaf
        previous_vertical_moment = token_leaf._get_vertical_moment()
        return previous_vertical_moment

    @property
    def start_components(self):
        r'''Tuple of components in vertical moment starting with at vertical
        moment, ordered by score index.
        '''
        result = []
        for component in self.components:
            if component._get_timespan().start_offset == self.offset:
                result.append(component)
        result = tuple(result)
        return result

    @property
    def start_leaves(self):
        r'''Tuple of leaves in vertical moment starting with vertical moment,
        ordered by score index.
        '''
        from abjad.tools import scoretools
        result = [x for x in self.start_components
            if isinstance(x, scoretools.Leaf)]
        result = tuple(result)
        return result

    @property
    def start_notes(self):
        r'''Tuple of notes in vertical moment starting with vertical moment,
        ordered by score index.
        '''
        from abjad.tools import scoretools
        result = [x for x in self.start_components
            if isinstance(x, scoretools.Note)]
        result = tuple(result)
        return result



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_logical_voice_01():
    r'''Yields nothing when class not present.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'4 c'4 c'4")
    staff = Staff(3 * tuplet)
    logical_voice = inspect_(staff[-1]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Rest,
        logical_voice,
        reverse=True,
        )

    assert len(list(iterator)) == 0


def test_agenttools_IterationAgent_by_logical_voice_02():
    r'''Yields internal nodes only.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'4 c'4 c'4")
    staff = Staff(3 * tuplet)
    logical_voice = inspect_(staff[-1]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Tuplet,
        logical_voice,
        reverse=True,
        )

    assert len(list(iterator)) == 3


def test_agenttools_IterationAgent_by_logical_voice_03():
    r'''Yields exact leaves.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'4 c'4 c'4")
    staff = Staff(3 * tuplet)
    logical_voice = inspect_(staff[-1]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Note,
        logical_voice,
        reverse=True,
        )

    assert len(list(iterator)) == 9


def test_agenttools_IterationAgent_by_logical_voice_04():
    r'''Yields leaves based on names higher in inheritence hierarchy.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'4 c'4 c'4")
    staff = Staff(3 * tuplet)
    logical_voice = inspect_(staff[-1][-1]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        scoretools.Leaf,
        logical_voice,
        reverse=True,
        )

    assert len(list(iterator)) == 9


def test_agenttools_IterationAgent_by_logical_voice_05():
    r'''Yields notes in two contiguous Voices with the same name.
    '''

    voice_1 = Voice("c'4 c'4")
    voice_2 = Voice("d'4 d'4")
    voice_1.name = voice_2.name = 'piccolo'
    staff = Staff([voice_1, voice_2])
    logical_voice = inspect_(staff[-1]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Note,
        logical_voice,
        reverse=True,
        )
    iterator = list(iterator)

    assert len(iterator) == 4
    for note in iterator:
        assert isinstance(note, Note)


def test_agenttools_IterationAgent_by_logical_voice_06():
    r'''Yields only notes matching the given logical voice.
    '''

    voice_1 = Voice("c'4 c'4")
    voice_2 = Voice("d'4 d'4")
    staff = Staff([voice_1, voice_2])
    logical_voice = inspect_(staff[-1]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Note,
        logical_voice,
        reverse=True,
        )
    iterator = list(iterator)

    assert len(iterator) == 2
    for note in iterator:
        assert isinstance(note, Note)
        assert note.written_pitch.numbered_pitch == 2


def test_agenttools_IterationAgent_by_logical_voice_07():
    r'''Yields only notes matching the given logical voice.
    '''

    voice_1 = Voice("c'4 c'4")
    voice_2 = Voice("d'4 d'4")
    voice_1.name = 'flute'
    voice_2.name = 'piccolo'
    staff = Staff([voice_1, voice_2])
    logical_voice = inspect_(staff[-1]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Note,
        logical_voice,
        reverse=True,
        )
    iterator = list(iterator)

    assert len(iterator) == 2
    for e in iterator:
        assert isinstance(e, Note)
        assert e.written_pitch.numbered_pitch == 2


def test_agenttools_IterationAgent_by_logical_voice_08():
    r'''Yields nothing when class not present.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'4 c'4 c'4")
    staff = Staff(3 * tuplet)
    logical_voice = inspect_(staff[0]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Rest,
        logical_voice,
        )

    assert len(list(iterator)) == 0


def test_agenttools_IterationAgent_by_logical_voice_09():
    r'''Yields internal nodes only.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'4 c'4 c'4")
    staff = Staff(3 * tuplet)
    logical_voice = inspect_(staff[0]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Tuplet,
        logical_voice,
        )
    assert len(list(iterator)) == 3


def test_agenttools_IterationAgent_by_logical_voice_10():
    r'''Yields exact leaves.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'4 c'4 c'4")
    staff = Staff(3 * tuplet)
    logical_voice = inspect_(staff[0]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Note,
        logical_voice,
        )

    assert len(list(iterator)) == 9


def test_agenttools_IterationAgent_by_logical_voice_11():
    r'''Yields leaves based on names higher in inheritence hierarchy.
    '''

    tuplet = Tuplet(Multiplier(2, 3), "c'4 c'4 c'4")
    staff = Staff(3 * tuplet)
    logical_voice = inspect_(staff[0][0]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        scoretools.Leaf,
        logical_voice,
        )

    assert len(list(iterator)) == 9


def test_agenttools_IterationAgent_by_logical_voice_12():
    r'''Yields notes in two contiguous Voices with the same name.
    '''

    voice_1 = Voice("c'4 c'4")
    voice_2 = Voice("d'4 d'4")
    voice_1.name = voice_2.name = 'piccolo'
    staff = Staff([voice_1, voice_2])
    logical_voice = inspect_(staff[0]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Note,
        logical_voice,
        )
    iterator = list(iterator)

    assert len(iterator) == 4
    for note in iterator:
        assert isinstance(note, Note)


def test_agenttools_IterationAgent_by_logical_voice_13():
    r'''Yields only notes matching the given logical voice.
    '''

    voice_1 = Voice("c'4 c'4")
    voice_2 = Voice("d'4 d'4")
    staff = Staff([voice_1, voice_2])
    logical_voice = inspect_(staff[0]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Note,
        logical_voice,
        )
    iterator = list(iterator)

    assert len(iterator) == 2
    for note in iterator:
        assert isinstance(note, Note)
        assert note.written_pitch.numbered_pitch == 0


def test_agenttools_IterationAgent_by_logical_voice_14():
    r'''Yields only notes matching the given logical voice.
    '''

    voice_1 = Voice("c'4 c'4")
    voice_2 = Voice("d'4 d'4")
    voice_1.name = 'flute'
    voice_2.name = 'piccolo'
    staff = Staff([voice_1, voice_2])
    logical_voice = inspect_(staff[0]).get_parentage().logical_voice
    iterator = iterate(staff).by_logical_voice(
        Note,
        logical_voice,
        )
    iterator = list(iterator)

    assert len(iterator) == 2
    for note in iterator:
        assert isinstance(note, Note)
        assert note.written_pitch.numbered_pitch == 0



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Note___repr___01():
    r'''Note repr is evaluable.
    '''

    note_1 = Note("c'4")
    note_2 = eval(repr(note_1))

    assert isinstance(note_1, Note)
    assert isinstance(note_2, Note)
    assert format(note_1) == format(note_2)
    assert note_1 is not note_2



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSet___contains___01():
    r'''PitchClassSet containment works as expected.
    '''

    named_pitch_class_set = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),])

    assert pitchtools.NamedPitchClass('c') in named_pitch_class_set
    assert pitchtools.NamedPitchClass('d') in named_pitch_class_set
    assert pitchtools.NamedPitchClass('e') in named_pitch_class_set

    assert not pitchtools.NamedPitchClass('cs') in named_pitch_class_set
    assert not pitchtools.NamedPitchClass('cf') in named_pitch_class_set


def test_pitchtools_PitchClassSet___contains___02():
    r'''PitchClassSet containment works as expected.
    '''

    pitch_class_set = pitchtools.PitchClassSet([0, 2, 6, 7])
    pitch_class_1 = pitchtools.NumberedPitchClass(2)
    pitch_class_2 = pitchtools.NumberedPitchClass(3)

    assert pitch_class_1 in pitch_class_set
    assert pitch_class_2 not in pitch_class_set



# -*- coding: utf-8 -*-


def make_reference_manual_graphviz_graph(graph):
    r'''Make a GraphvizGraph instance suitable for use in the
    Abjad reference manual.

    Returns GraphvizGraph instance.
    '''
    from abjad.tools import graphtools
    if not isinstance(graph, graphtools.GraphvizGraph):
        try:
            graph = graph.__graph__()
        except:
            return graph
    graph.attributes['size'] = "8.0, 12.0"
    graph.node_attributes['fontsize'] = 10
    return graph



# -*- coding: utf-8 -*-


def is_integer_singleton(argument):
    r'''Is true when `argument` is an integer tuple of of length 1. Otherwise
    false.

    ..  container:: example

        ::

            >>> mathtools.is_integer_singleton((19,))
            True

        ::

            >>> mathtools.is_integer_singleton(('text',))
            False

    Returns true or false.
    '''
    return (
        isinstance(argument, tuple) and
        len(argument) == 1 and
        isinstance(argument[0], int)
        )
